@@ -1878,6 +1878,14 @@ bool needsReReturning(llvm::Argument *arg, size_t &sret_idx,
   return true;
 }
 
+static bool isOpaque(llvm::Type *T) {
+#if LLVM_VERSION_MAJOR >= 20
+  return T->isPointerTy();
+#else
+  return T->isOpaquePointerTy();
+#endif
+}
+
 // TODO, for sret/sret_v check if it actually stores the jlvalue_t's into the
 // sret If so, confirm that those values are saved elsewhere in a returnroot
 void EnzymeFixupJuliaCallingConvention(LLVMValueRef F_C,
@@ -2108,7 +2116,7 @@ void EnzymeFixupJuliaCallingConvention(LLVMValueRef F_C,
   SmallVector<Type *, 1> types;
   size_t nexti = 0;
   if (sretTy) {
-    types.push_back(PointerType::getUnqual(sretTy));
+    types.push_back(getUnqual(sretTy));
     NewAttrs = NewAttrs.addAttribute(
         F->getContext(), AttributeList::FirstArgIndex + nexti,
         Attribute::get(F->getContext(), Attribute::StructRet, sretTy));
@@ -2127,7 +2135,7 @@ void EnzymeFixupJuliaCallingConvention(LLVMValueRef F_C,
     NewAttrs = NewAttrs.addAttribute(F->getContext(),
                                      AttributeList::FirstArgIndex + nexti,
                                      Attribute::WriteOnly);
-    types.push_back(PointerType::getUnqual(roots_AT));
+    types.push_back(getUnqual(roots_AT));
     nexti++;
   }
 
@@ -2301,8 +2309,7 @@ void EnzymeFixupJuliaCallingConvention(LLVMValueRef F_C,
           }
           if (subCount != numRooting) {
             gep = EB.CreatePointerCast(
-                gep,
-                PointerType::getUnqual(ArrayType::get(T_prjlvalue, subCount)));
+                gep, getUnqual(ArrayType::get(T_prjlvalue, subCount)));
           }
           curOffset += subCount;
           if (reret_roots.count(i))
@@ -2340,6 +2347,8 @@ void EnzymeFixupJuliaCallingConvention(LLVMValueRef F_C,
     assert(sretCount == Types.size());
   }
 
+  auto &DL = F->getParent()->getDataLayout();
+
   // TODO fix caller side
   for (auto CI : callers) {
     auto Attrs = CI->getAttributes();
@@ -2401,7 +2410,10 @@ void EnzymeFixupJuliaCallingConvention(LLVMValueRef F_C,
 
           bool handled = false;
           if (auto AI = dyn_cast<AllocaInst>(getBaseObject(val, false))) {
-            if (AI->getAllocatedType() == Types[sretCount]) {
+            if (AI->getAllocatedType() == Types[sretCount] ||
+                (isOpaque(AI->getType()) &&
+                 DL.getTypeSizeInBits(AI->getAllocatedType()) ==
+                     DL.getTypeSizeInBits(Types[sretCount]))) {
               AI->replaceAllUsesWith(gep);
               AI->eraseFromParent();
               handled = true;
@@ -2425,8 +2437,9 @@ void EnzymeFixupJuliaCallingConvention(LLVMValueRef F_C,
             }
 
             postCallReplacements.emplace_back(val, gep, Types[sretCount],
-                                              sret_jlvalue);
-            preCallReplacements.emplace_back(val, gep, Types[sretCount]);
+                                              should_sret);
+            if (!isWriteOnly(CI, i))
+              preCallReplacements.emplace_back(val, gep, Types[sretCount]);
           }
 
           if (roots_AT && reroot_enzyme_srets.count(i)) {
@@ -2459,8 +2472,7 @@ void EnzymeFixupJuliaCallingConvention(LLVMValueRef F_C,
 
             if (subCount != numRooting) {
               gep = AIB.CreatePointerCast(
-                  gep, PointerType::getUnqual(
-                           ArrayType::get(T_prjlvalue, subCount)));
+                  gep, getUnqual(ArrayType::get(T_prjlvalue, subCount)));
             }
             local_root_count += subCount;
             if (reret_roots.count(i))
