@@ -195,6 +195,9 @@ public:
                 const llvm::Function *CodeRegion);
 };
 
+// Forward declaration needed for EmitFailure template
+llvm::Function *getFirstFunctionDefinition(llvm::Module &M);
+
 template <typename... Args>
 void EmitFailure(llvm::StringRef RemarkName,
                  const llvm::DiagnosticLocation &Loc,
@@ -217,6 +220,21 @@ void EmitFailure(llvm::StringRef RemarkName,
       (EnzymeFailure("Enzyme: " + ss.str(), Loc, CodeRegion)));
 }
 
+template <typename... Args>
+void EmitFailure(llvm::StringRef RemarkName, llvm::Module &M, Args &...args) {
+  // Use the first function definition in the module as context for the
+  // diagnostic
+  if (llvm::Function *FirstFunc = getFirstFunctionDefinition(M)) {
+    EmitFailure(RemarkName, FirstFunc->getSubprogram(), FirstFunc, args...);
+  } else {
+    // Fallback if no functions in module
+    std::string *str = new std::string();
+    llvm::raw_string_ostream ss(*str);
+    (ss << ... << args);
+    llvm::report_fatal_error(llvm::StringRef(*str));
+  }
+}
+
 static inline llvm::Function *isCalledFunction(llvm::Value *val) {
   if (llvm::CallInst *CI = llvm::dyn_cast<llvm::CallInst>(val)) {
     return CI->getCalledFunction();
@@ -1160,6 +1178,14 @@ static inline llvm::PointerType *getInt8PtrTy(llvm::LLVMContext &Context,
 #endif
 }
 
+static inline llvm::PointerType *getUnqual(llvm::Type *T) {
+#if LLVM_VERSION_MAJOR >= 17
+  return llvm::PointerType::getUnqual(T->getContext());
+#else
+  return llvm::PointerType::getUnqual(T);
+#endif
+}
+
 static inline llvm::StructType *getMPIHelper(llvm::LLVMContext &Context) {
   using namespace llvm;
   auto i64 = Type::getInt64Ty(Context);
@@ -1395,6 +1421,8 @@ void ErrorIfRuntimeInactive(llvm::IRBuilder<> &B, llvm::Value *primal,
 
 llvm::Function *GetFunctionFromValue(llvm::Value *fn);
 
+llvm::Function *getFirstFunctionDefinition(llvm::Module &M);
+
 llvm::Value *simplifyLoad(llvm::Value *LI, size_t valSz = 0,
                           size_t preOffset = 0);
 
